# -*- coding:utf-8 -*-
"""
    auto doc tools
    author comger@gmail.com
    function:
        kpages_init.py --doc all     : doc all case for app
    demos:
        kpages_init.py --doc all                          :doc methods in app's __conf__.UTEST_DIR

    doc dir is config in __conf__.UTEST_DIR
"""
import os
import sys
import profile
from fnmatch import fnmatch
from inspect import isclass, ismethod, getmembers

from kpages.utility import _get_members, app_path
from kpages.router import load_handlers


def _load_api(handler_dir='action', member_filter=None):
    '''
    Load handler's url api 
    '''
    path = app_path(handler_dir)
    sys.path.append(os.getcwd())
    py_filter = lambda f: fnmatch(f, '*.py') and not f.startswith('__')

    if not member_filter:
        member_filter = lambda m: isinstance(
            m, type) and hasattr(m, '__urls__') and m.__urls__

    names = [os.path.splitext(n)[0] for n in os.listdir(path) if py_filter(n)]
    modules = [__import__(
        "{0}.{1}".format(handler_dir, n)).__dict__[n] for n in names]

    mts = ('get', 'post', 'put', 'patch', 'delete')

    rets = []
    for m in modules:        
        apis = []
        doc = {'module': m, 'apis':apis}

        for k, v in getmembers(m, member_filter):
            
            print(k, v, v.__module__)
            for _method in mts:
                _mt = getattr(v, _method)
                #import pdb;pdb.set_trace()
                #todo if method in private for subclass 
                if _mt and _mt.__doc__:
                    mt_doc = dict(method=_method, url=v.__urls__, doc=_mt.__doc__, func=_mt)
                    apis.append(mt_doc)

        rets.append(doc)
    return rets

def load_api(handler_dir='action', member_filter=None):
    ''' load all handler in dirs '''
    dirs = handler_dir
    if isinstance(handler_dir, str):
        dirs = (handler_dir,)
    handlers = []
    for path in dirs:
        handlers.extend(_load_api(path, member_filter=member_filter))

    return handlers


def api2markdown(rets):
    ''' api to markdown '''
    header = """
## {0}
模块说明 {1}
### 接口清单
"""

    api_format = """
{0}. [{1}] {2}
```
{3}
```
    """

    doc = ''
    num = 1
    for _doc in rets:
        _header = header.format(_doc['module'].__name__, str(_doc['module'].__doc__).replace('\t',''))
        for api in _doc['apis']:
            pre_url = [ url[0] for url in api['url']]
            _header = _header+api_format.format(num,api['method'].upper(), ','.join(pre_url), str(api['doc']).replace('\t',''))
            num = num +1
        
        doc = doc + _header
    
    
    f = open(app_path('apis.md'),'w')
    f.write(doc)
    f.close()

def run_doc(line=None):
    _handlers = load_api(__conf__.ACTION_DIR)
    api2markdown(_handlers)
    print('auto gen apis.md done!')


__all__ = ['run_doc']



# -*- coding:utf-8 -*-

"""
    系统配置文件

    不建议修改本文件，应用设置应该保存在 /settings.py 应用配置文件中。
    应用配置会覆盖同名的系统配置。

"""

# 服务器监听端口
PORT = 8088

# http accept ip 
BIND_IP = None

# 数据库超时时间
SOCK_TIMEOUT = 10
SOCK_TIMEOUT_MS = None

CPU_MULTIPLE = 5

# SESSION 过期时间(秒)
SESSION_EXPIRE = 30*24*60*60

# 是否启动调试模式
DEBUG = True

# 是否启动压缩
GZIP = True

# 指定RequestHandler 所在目录
ACTION_DIR = ("action",)

# 指定任务（srvcmd）所在目录
JOB_DIR = "logic"

# 指定TestCase 所在目录
UTEST_DIR = "utest"

# 静态目录名
STATIC_DIR_NAME = "static"

# 模板目录名
TEMPLATE_DIR_NAME = "template"

# COOKIE 加密
COOKIE_SECRET = "61oETzKXQAGaYdkL5gEmGeJJFuYh7EQnp2XdTP1o/Vo="

# COOKIE 安全
XSRF_COOKIES = False

# 数据库与缓存是否使用持久连接
PERSISTENT_DB_CONNECTION = False

# 缓存服务器
CACHE_HOST = "localhost"

# 数据库服务器
DB_HOST = "localhost"

# 数据库名
DB_NAME = "test"

GFS_NAME = DB_NAME

SERVICE_CHANNEL = "channel1"

SERVICE_LISTKEY = "kpages_cmd_list"

max_buffer_size = 104857600 * 10

RPC_PORT = 8080
RPC_DIR = ('rpc',)




#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
    Request router for tornado
    author comger@gmail.com
"""
import os
import sys
import tornado.web
from fnmatch import fnmatch
from inspect import getmembers
from kpages.utility import app_path


def reg_ui_method(name=None, intro=None):
    """
    reg ui_method
    Demo
    @reg_ui_method()
    def add(self,a,b,c):
        return a+b+c

    """
    def actual(handler):
        ''' set attr for handler'''
        if not hasattr(handler, '__reg_ui__'):
            handler.__reg_ui__ = True
            handler.__uiname__ = name
            handler.__intro__ = intro

        return handler

    return actual


def url(pattern=None, order=0):
    """
    set router for RequestHandler 

    Demo:
    @url('/blog/info/(.*)')
    class ActionHandler(tornado.web.RequestHandler):
        def get(self,blogid):
            pass
    """
    def actual(handler):
        ''' set attr for handler'''
        assert(issubclass(handler, tornado.web.RequestHandler))
        if not hasattr(handler, "__urls__") or not handler.__urls__:
            handler.__urls__ = []

        if not pattern:
            p = '/{0}/{1}'.format(handler.__module__, handler.__name__).lower()
            p = p.replace('.', '/')
            handler.__urls__.append((p, 0))
        else:
            handler.__urls__.append((pattern, order))

        return handler

    return actual


def load_handlers(handler_dir='action', member_filter=None):
    ''' load all handler in dirs '''
    dirs = handler_dir
    if isinstance(handler_dir, str):
        dirs = (handler_dir,)
    handlers = []
    for path in dirs:
        handlers.extend(_load_handlers(path, member_filter=member_filter))

    return handlers


def _load_handlers(handler_dir='action', member_filter=None):
    '''
        Load handler_dir's Handler
        Demo:
        handlers = load_handlers():
        app = tornado.web.Application(handlers)
    '''
    #path = os.path.join(os.getcwd(), handler_dir)
    path = app_path(handler_dir)
    sys.path.append(os.getcwd())
    py_filter = lambda f: fnmatch(f, '*.py') and not f.startswith('__')

    if not member_filter:
        member_filter = lambda m: isinstance(
            m, type) and hasattr(m, '__urls__') and m.__urls__

    names = [os.path.splitext(n)[0] for n in os.listdir(path) if py_filter(n)]
    modules = [__import__(
        "{0}.{1}".format(handler_dir, n)).__dict__[n] for n in names]

    ret = {}
    for m in modules:
        members = dict(("{0}.{1}".format(
            v.__module__, k), v) for k, v in getmembers(m, member_filter))
        ret.update(members)
    return _sorted_hanlders(ret.values())



def _sorted_hanlders(handlers):
    ''' sort handlers '''
    handlers = [(pattern, order, h) for h in handlers for pattern,
                order in h.__urls__]

    handlers.sort(key=lambda x: x[1])
    return [(pattern, handler) for pattern, _, handler in handlers]


__all__ = ["reg_ui_method", "url", "load_handlers"]



# -*- coding:utf-8 -*-
"""
    author comger@gmail.com
    logic context for db, cache
    db is mongodb,cache is redis
"""
import os
import time
import tornado.gen as gen

from threading import local
from hashlib import sha1

from tornado.web import RequestHandler
from tornado.websocket import WebSocketHandler

try:
    from pymongo import MongoClient
    from pymongo.errors import AutoReconnect
    from gridfs import GridFS
except:
    print("pymongo is no exist!")

try:
    from redis import Redis
except:
    print("Redis is no exist!")

try:
    from motor.motor_tornado import MotorClient
except:
    print("motor is no exist!")

session_id = lambda: sha1('%s%s' % (os.urandom(16), time.time())).hexdigest()


class ContextHandler(object):
    """
    base handler for session
    """
    def _execute(self, transforms, *args, **kwargs):
        ''' select base handler for self '''
        try:
            with LogicContext():
                get_context().handler = self
                if isinstance(self, WebSocketHandler):
                    WebSocketHandler._execute(self, transforms, *args, **kwargs)
                elif isinstance(self, RequestHandler):
                    RequestHandler._execute(self, transforms, *args, **kwargs)
        except AutoReconnect as e:
            print(e)

    def session(self, key, val=None):
        ''' session for handler '''
        return get_context().session(self.get_redis_key(key), val)


    def clear_session(self, key):
        get_context().clear_session(self.get_redis_key(key))


    def get_redis_key(self, key):
        _id = self.get_secure_cookie('session_id', None)
        if not _id:
            _id = session_id()
            self.set_secure_cookie('session_id', _id)
        return '{0}.{1}'.format(_id, key)
        

class LogicContext(object):
    """
        logic context for mongodb and redis cache
    """
    _thread_local = local()

    def __init__(self, redis_host=None, mongo_host=None):
        self._redis_host = redis_host or __conf__.CACHE_HOST
        self._mongo_host = mongo_host or __conf__.DB_HOST
        self._db_conn = None
        self._cache = None
        self._motor_clt = None

    def __enter__(self):
        if not hasattr(self._thread_local, "contexts"):
            self._thread_local.contexts = []
        self._thread_local.contexts.append(self)
        return self

    def __exit__(self, exc_type, exc_value, trackback):

        self._thread_local.contexts.remove(self)
        if self._db_conn:
            self._db_conn.disconnect()

    @classmethod
    def get_redis(cls):
        ''' get redis from context '''
        host = __conf__.CACHE_HOST
        h, p = host.split(":") if ":" in host else (host, 6379)

        if not hasattr(cls,'__cache__') or (cls.__cache__ and not cls.__cache__.ping()):
            cls.__cache__ = Redis(
                host=h, port=int(p), socket_timeout=__conf__.SOCK_TIMEOUT)

        return cls.__cache__

    def get_gfs(self, name=None):
        ''' get gfs from context '''
        name = name or __conf__.GFS_NAME
        return GridFS(self.get_mongoclient(name = name))


    def get_motor(self, name=None):
        ''' motor client '''
        name = name or __conf__.DB_NAME
        if not self._motor_clt:
            self._motor_clt = MotorClient(host=__conf__.DB_HOST)

        return self._motor_clt[name]


    def session(self, key, val=None, expire= None):
        '''
        redis session for tornado
        '''
        expire = expire or __conf__.SESSION_EXPIRE
        return self.get_redis().setex(key, val, expire) if val else self.get_redis().get(key)


    def clear_session(self, key):
        self.get_redis().delete(key)


    @classmethod
    def get_context(cls, **kwargs):
        return hasattr(cls._thread_local, "contexts") and cls._thread_local.contexts and \
            cls._thread_local.contexts[-1] or cls(mongo_host=kwargs.get('db_host'))

    @classmethod 
    def get_mongoclient(cls, name=None):
        """
        get mongoclient in application
        """
        name = name or __conf__.DB_NAME
        if not hasattr(cls,'__mongoclient__'):
            cls.__mongoclient__ = MongoClient(host = __conf__.DB_HOST,
                socketTimeoutMS = __conf__.SOCK_TIMEOUT_MS)

        return cls.__mongoclient__[name]
    

    @classmethod
    def get_replicaset(cls, hosts=None, replicaSet=None):
        hosts = hosts or __conf__.ReplicaSetHost
        replicaSet = replicaSet or __conf__.replicaSet

        if not hasattr(cls, '__replicaset__'):
            cls.__replicaset__ = MongoReplicaSetClient(hosts, replicaSet=relicaSet)

        return cls.__replicaset__


get_context = LogicContext.get_context

__all__ = ["ContextHandler", "LogicContext", "get_context"]



#!/usr/bin/env python
#-*- coding:utf-8 -*-

"""
    系统队列处理服务

    通常和 Redis 部署在同一台服务器上，也可以通过命令行参数指定(--help 查看所有命令行参数)。

    历史:
        2011-08-27  + 增加 optparse 处理命令行参数。
"""

from pprint import pprint
from optparse import OptionParser, OptionGroup
from kpages import Service, reflesh_config


def _show_info(**kwargs):

    services = kwargs.pop("services")
    settings = kwargs

    print("MQ Service, version 0.9.2011.08")


    print("Parameters:")
    for k in sorted(settings.keys()):
        if k.startswith("__"):
            continue
        print("  {0:<20} : {1}".format(k.upper(), settings[k]))



    print("Services:")
    pprint(services)



def _get_opt():
    parser = OptionParser("%prog [options]", version="%prog v0.9")
    parser.add_option("--config", dest="config",
                      default='setting.py', help="config for app")
    parser.add_option("--host", dest="host",
                      default=None, help="Redis server address.")
    parser.add_option("--channel", dest="channel",
                      default=None, help="Subscribe channel name.")
    parser.add_option("--debug", dest="debug",
                      action="store_true", default=None, help="Debug mode.")
    parser.add_option("--ndebug", dest="debug",
                      action="store_false", help="No Debug mode.")

    return parser.parse_args()


def main():
    opts, args = _get_opt()
    reflesh_config(opts.config)
    if opts.debug is not None:
        __conf__.DEBUG = opts.debug
    
    Service(host=__conf__.CACHE_HOST or opts.host, channel=__conf__.SERVICE_CHANNEL or opts.channel,
            callback=_show_info).run()


if __name__ == "__main__":
    main()

# vim: ts=4 sw=4 sts=4 expandtab



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
    MinxiHandler for kpages
    author comger@gmail.com
    1. auth
    
"""
import json
import tornado
from utility import mongo_conv 
try:
    import urlparse
except ImportError:
    import urllib.parse as urlparse

AUTH_KEY = 'authed_user'

class AuthMinix(object):
    uid = property(lambda self:self.current_user['_id'])

    def get_current_user(self):
        user_json = self.get_secure_cookie(AUTH_KEY)
        if not user_json: return None
        return json.loads(user_json)
        
    def prepare(self):
        if not self.current_user:
            url = __conf__.login_url
            if "?" not in url:
                if urlparse.urlsplit(url).scheme:
                    next_url = self.request.full_url()
                else:
                    next_url = self.request.uri

                url += "?" + urlencode(dict(next=next_url))
            self.redirect(url)
            return

    def login(self, user):
        user = mongo_conv(user)
        self.set_secure_cookie(AUTH_KEY, json.dumps(user))





#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
    author comger@gmail.com
"""
import tornado.web
import tornado.ioloop
import tornado.options

from tornado.options import define, options
from inspect import isclass
from tornado.httpserver import HTTPServer
from optparse import OptionParser, OptionGroup
from kpages.router import load_handlers
from kpages.context import LogicContext
from kpages.utility import refresh_config, app_path, set_default_encoding, get_members

def get_ui_modules():
    """
    return ui module members in ACTION_DIR
    """
    m_filter = lambda m: isclass(m) and issubclass(m, tornado.web.UIModule)
    ms =  get_members(__conf__.ACTION_DIR, member_filter=m_filter)
    if 'tornado.web.UIModule' in ms:
        del ms['tornado.web.UIModule']
    
    newms = {}
    for key,val in ms.items():
        newms[key.replace('.','_')] = val
    
    return newms

def get_ui_methods():
    """
    return uimethod methods in ACTION_DIR
    """
    m_filter = lambda m: hasattr(m,'__reg_ui__') and m.__reg_ui__ ==True
    ms =  get_members(__conf__.ACTION_DIR, member_filter=m_filter)
    newms = {}

    for key,val in ms.items():
        newms[key.replace('.','_')] = val
    
    return newms

class WebApp(object):
    settings = property(lambda self: __conf__.__dict__)
    handlers = property(lambda self: self._handlers)
    uimodules = property(lambda self: self._modules)
    uimethods = property(lambda self: self._methods)
    webapp = property(lambda self: self._webapp)

    def __init__(self, port=None,bindip=None, handlers=None, callback=None):
        self._port = port or __conf__.PORT
        self._ip = bindip or __conf__.BIND_IP
        self._callback = callback
        self._handlers = load_handlers(__conf__.ACTION_DIR)
        self._modules = get_ui_modules()
        self._methods = get_ui_methods()
        self._webapp = self._get_webapp()
        __conf__.APP = self._webapp    

    def _get_webapp(self):
        settings = {"debug": __conf__.DEBUG,
                    "static_path": app_path(__conf__.STATIC_DIR_NAME),
                    "template_path": app_path(__conf__.TEMPLATE_DIR_NAME),
                    "gzip": __conf__.GZIP,
                    "cookie_secret": __conf__.COOKIE_SECRET,
                    "ui_modules":self.uimodules,
                    "ui_methods":self.uimethods,
                    "xsrf_cookies": __conf__.XSRF_COOKIES}

        
        return tornado.web.Application(self._handlers, **settings)


    def _run_server(self):
        if __conf__.DEBUG:
            print('web app')
            self._webapp.listen(self._port, address=self._ip, max_buffer_size = __conf__.max_buffer_size)
        else:
            server = HTTPServer(self._webapp, xheaders=True, max_buffer_size = __conf__.max_buffer_size)
            server.bind(self._port, address=self._ip)
            server.start(0)
        #tornado.ioloop.IOLoop.instance().start()
        tornado.ioloop.IOLoop.current().start()

    def run(self):
        if self._callback:
            self._callback(self)
        self._run_server()


define("port", default=None, help="run on the given port", type=int)
define("config", default='setting.py', help="set config for server")
define("ip",  help="bind accept ip  for server")
define("debug", default=None,  help="Debug Mode")
define("ndebug",  help="No Debug Mode")


def run(callback=None):
    set_default_encoding()
    tornado.options.parse_command_line()
    opts = options
    refresh_config(opts.config)

    __conf__.PORT = opts.port or __conf__.PORT
    __conf__.DEBUG = opts.debug or __conf__.DEBUG
    __conf__.BIND_IP = opts.ip or __conf__.BIND_IP

    WebApp(callback=callback).run()

__all__ = ["run", "WebApp", "get_ui_modules", "get_ui_methods"]



#!/usr/bin/env python
#
# Copyright 2009 comger@gmail.com
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import distutils.core
import sys
# Importing setuptools adds some features like "setup.py develop", but
# it's optional so swallow the error if it's not there.
try:
    import setuptools
except ImportError:
    pass

kwargs = {}


version = "2.0.0"

class PyTest(distutils.core.Command):
    user_options = []
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        import os, sys, unittest
        setup_file = sys.modules['__main__'].__file__
        setup_dir = os.path.abspath(os.path.dirname(setup_file))
        test_loader = unittest.defaultTestLoader
        test_runner = unittest.TextTestRunner()
        test_suite = test_loader.discover(setup_dir)
        test_runner.run(test_suite)


distutils.core.setup(
    name="kpages",
    version=version,
    packages=["kpages"],
    package_data={'': ['*.*']},
    author="comger@gmail.com",
    author_email="comger@gmail.com",
    url="http://github.com/comger/kpages",
    license="http://www.apache.org/licenses/LICENSE-2.0",
    description="kpages is api-centric web application framework for the Tornado and Mongodb",
    scripts=['kpages/kpages_init.py', 'kpages/kpages_web.py', 'kpages/kpages_service.py', 'kpages/kpages_tool.py', 'kpages/kpages_rpc.py'],
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
    keywords=["pymongo", "gridfs", "profile", "tornado", "redis", "session", "router", "MQ"],
    cmdclass={'test': PyTest},
    **kwargs
)



# -*- coding:utf-8 -*-
"""
    auto test tools
    author comger@gmail.com
    function:
        python app.py --test all     : test all case for app
        python app.py --test line    : test the methods in line path
        python app.py --pro  line    : profile the methods in line path(demo like --test)
    demos:
        python app.py --test test_city.DemoCase.testprint :test testprint method
        python app.py --test test_city.DemoCase           :test methods in DemoCase class
        python app.py --test test_city                    :test methods in test_city.py
        python app.py --test all                          :test methods in app's __conf__.UTEST_DIR

    utest dir is config in __conf__.UTEST_DIR
"""
import os
import profile
from inspect import isclass, ismethod, getmembers
from unittest import TestCase, TextTestRunner, TestSuite
from kpages.utility import _get_members

def load_testcase(module=None):
    _filter = lambda m: isclass(m) and issubclass(m, TestCase) and not m==TestCase
    testcases = _get_members(
        __conf__.UTEST_DIR, member_filter=_filter, in_module=module)

    return testcases

def load_testsuites(module=None):
    """ load all test cases in module, if module==None,all test cases in utest dir """
    testcases = load_testcase(module)
    _suites = {}
    for name, cls in testcases.items():
        for n, m in getmembers(cls):
            if n.startswith("test") and ismethod(m):
                _suites["{0}.{1}".format(name, n)] = TestSuite((cls(n),))

    return _suites


def load_testsuites_bypath(line=None):
    _suites = []
    if not line:
        _suites = load_testsuites().values()
    else:
        ls = line.strip().split('.')
        suites = load_testsuites(ls[0])
        if len(ls) == 2:
            for key in suites.keys():
                if key.startswith('utest.{0}.{1}'.format(ls[0], ls[1])):
                    _suites.append(suites[key])
        elif len(ls) == 3:
            _suites = suites.get('utest.' + line)
        elif len(ls) == 1:
            _suites = suites.values()
    
    return _suites


def run_test(line=None):
    _suites = load_testsuites_bypath(line)

    print("Unittest:")
    print(_suites)
    TextTestRunner().run(TestSuite(_suites))


def pro_test(m):
    def _run():
        run_test(m)
    profile.runctx("_run()", globals(), locals())


__al__ = ['run_test', 'pro_test']



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
    author comger@gmail.com
    kpages tool for init project 
"""
import sys
import os
import zipfile
import kpages
from kpages import app_path

if __name__ == '__main__':
    path = 'kpages_project'
    if len(sys.argv)>1:
        path = sys.argv[1]

    root = kpages.__path__[0]
    from_path = root +'/web.zip'
    to_path = app_path(path)
    _file = zipfile.ZipFile(from_path)
    _file.extractall(to_path)
    print('init project at:',to_path)



# -*- coding:utf-8 -*-
"""
    author comger@gmail.com
"""
import os
import sys
import imp
import datetime
import builtins
from types import ModuleType
from bson.objectid import ObjectId
from fnmatch import fnmatch
from inspect import getmembers

#merge f with current path
app_path = lambda f: os.path.join(os.getcwd(), f)

try:
    from line_profiler import LineProfiler
    def lpro(follow=[]):
        def inner(func):
            def profiled_func(*args, **kwargs):
                try:
                    profiler = LineProfiler()
                    profiler.add_function(func)
                    for f in follow:
                        profiler.add_function(f)

                    profiler.enable_by_count()
                    return func(*args, **kwargs)
                finally:
                    profiler.print_stats()
            return profiled_func
        return inner

except ImportError:
    def lpro(follow=[]):
        def inner(follow=[]):
            def nothing(*args, **kwargs):
                return func(*args, **kwargs)
            return nothing

        return inner


def get_modules(m_path):
    ''' get all py module in m_path '''
    path = app_path(m_path)
    py_filter = lambda f: fnmatch(f, '*.py') and not f.startswith('__')
    names = [os.path.splitext(n)[0] for n in os.listdir(path) if py_filter(n)]
    sys.path.append(os.getcwd())
    return [__import__("{0}.{1}".format(m_path, n)).__dict__[n] for n in names]


def _get_members(m_path, member_filter=None, in_module=None):
    ''' get all members in m_path for member_filter'''
    modules = get_modules(m_path)
    if not member_filter:
        member_filter = lambda m: isinstance(m, type)

    if in_module:
        m = __import__("{0}.{1}".format(m_path, in_module)).__dict__[in_module]
        return dict(("{0}.{1}".format(v.__module__, k), v) for k, v in getmembers(m, member_filter))

    ret = {}
    for m in modules:
        members = dict(("{0}.{1}".format(
            v.__module__, k), v) for k, v in getmembers(m, member_filter))
        ret.update(members)
    return ret


def get_members(dirs, member_filter=None):
    if isinstance(dirs, str):
        dirs = (dirs,)

    ms = {}
    for path in dirs:
        try:
            ms.update(_get_members(path,member_filter=member_filter))
        except Exception as e:
            print(e)

    return ms


def not_empty(*args):
    if not all(args):
        raise ValueError("Argument must be not None/Null/Zero/Empty!")


def refresh_config(*args):
    '''
        reflesh setting.py to builtins

        demo: reflesh_config('setting.py.txt','cacheconfig.py.txt')
        use like : __conf__.DB_HOST
    '''
    if not args:
        args = ()

    dct = {}
    from kpages import settings
    dct.update(settings.__dict__)

    pys = map(app_path, args)
    for py in pys:
        if os.path.exists(py):
            exec(compile(open(py, "rb").read(), py, 'exec'), dct)

    module = ModuleType("__conf__")
    for k, v in dct.items():
        if not k.startswith("__"):
            setattr(module, k, v)

    builtins.__conf__ = module

reflesh_config = refresh_config

def mongo_conv(d):
    if isinstance(d, (ObjectId, datetime.datetime)):
        return str(d)
    elif isinstance(d,(unicode,)):
        return str(d.encode('utf-8'))
    elif isinstance(d, list):
        return map(mongo_conv, d)
    elif isinstance(d, tuple):
        return tuple(map(mongo_conv, d))
    elif isinstance(d, dict):
        return dict([(mongo_conv(k), mongo_conv(v)) for k, v in d.items()])
    else:
        return d


def set_default_encoding():
    import sys
    imp.reload(sys)



__all__ = ["app_path", "not_empty", "refresh_config", "reflesh_config", "mongo_conv",
           "set_default_encoding", "get_modules", "get_members", "lpro"]